@@ -4,7 +4,7 @@
  * GChemPaint library
  * application.cc 
  *
- * Copyright (C) 2004-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -54,6 +54,7 @@
 #include "zoomdlg.h"
 #include <gcu/filechooser.h>
 #include <gcu/loader.h>
+#include <gcu/loader-error.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/reaction.h>
@@ -254,6 +255,11 @@ static Object* CreateMechanismStep ()
 	return new MechanismStep (MechanismStepType);
 }
 
+static Object* CreateElectron ()
+{
+	return new Electron (NULL, false);
+}
+
 bool	Application::m_bInit = false;
 bool	Application::m_Have_Ghemical = false;
 bool	Application::m_Have_InChI = false;
@@ -314,7 +320,8 @@ Application::Application ():
 		Object::AddType ("mesomery-arrow", CreateMesomeryArrow, MesomeryArrowType);
 		Object::AddType ("text", CreateText, TextType);
 		Object::AddType ("fragment", CreateFragment, FragmentType);
-		ElectronType = Object::AddType ("electron", NULL);
+		ElectronType = Object::AddType ("electron", CreateElectron);
+		Object::AddAlias (ElectronType, "electron-pair");
 		// Add rules
 		Object::AddRule ("reaction", RuleMustContain, "reaction-step");
 		Object::AddRule ("reaction-step", RuleMustContain, "reactant");
@@ -633,7 +640,7 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 			}
 		}
 		g_object_unref (file);
-	} else  { //loading
+	} else  try { //loading
 		file = g_file_new_for_uri (filename);
 		err = g_file_query_exists (file, NULL);
 		g_object_unref (file);
@@ -679,6 +686,7 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 			case ContentTypeSpectrum:
 				break;
 			default: {
+				pDoc->Loaded ();
 				double l = pDoc->GetMedianBondLength ();
 				if (l > 0.) {
 					double r = pDoc->GetBondLength () / l;
@@ -709,6 +717,20 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 				OpenWithBabel (filename2, mime_type, pDoc);
 		}
 	}
+	catch (LoaderError &e) {
+		pDoc->Clear ();
+		char *unescaped = g_uri_unescape_string (filename, NULL);
+		string mess = _("Error in ");
+		mess += unescaped;
+		// Note to translator: add a space if needed before the semicolon
+		mess += _(":\n");
+		mess += e.what ();
+		GtkWidget *message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess.c_str (), NULL, NULL);
+		g_free (unescaped);
+		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
+		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
+		gtk_widget_show (message);
+	}
 	return false;
 }
 
