@@ -4,7 +4,7 @@
  * Gnome Chemisty Utils
  * crystaldoc.h 
  *
- * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -33,11 +33,8 @@
 #include "crystalline.h"
 #include "crystalcleavage.h"
 #include "document.h"
+#include "macros.h"
 #include <gcu/gldocument.h>
-#ifdef HAVE_OPENBABEL_2_2
-#	include <openbabel/math/spacegroup.h>
-#	include <openbabel/mol.h>
-#endif
 
 /*!\file*/
 namespace gcu
@@ -45,6 +42,7 @@ namespace gcu
 
 class CrystalView;
 class Matrix;
+class SpaceGroup;
 
 /*!\enum gcLattices crystalviewer/crystaldoc.h
 This enumeration gives sympolic names to the fourteen Bravais lattices.
@@ -132,7 +130,7 @@ CrystalView.
 */
 	virtual CrystalView* CreateNewView ();
 /*!
-Creates a new atom. This method should be overrided by programs deriving a new view class from
+Creates a new atom. This method should be overrided by programs deriving a new atom class from
 CrystalAtom.
 
 @return a pointer to the new CrystalAtom instance.
@@ -163,20 +161,45 @@ compatiblity with files created by older versions of the program.
 */
 	virtual const char* GetProgramId () const;
 
-#ifdef HAVE_OPENBABEL_2_2
-	bool ImportOB (OpenBabel::OBMol &mol);
-#endif
-	
+/*!
+@param property the property id as defined in objprops.h
+@param value the property value as a string
+
+Used when loading to set properties to the document
+@return true if the property could be set, or if the property is not relevant, false otherwise.
+*/
+	bool SetProperty (unsigned property, char const *value);
+
+/*!
+@param property the property id as defined in objprops.h
+
+Used when saving to get properties from the document.
+@return the property as a string. The returned string might be empty.
+*/
+	std::string GetProperty (unsigned property) const;
+
+/*!
+Called by the application whe the document has been loaded to update the title
+and add some lines.
+*/
+	bool Loaded () throw (LoaderError);
+/*!
+	@param object the Object instance to add as a child.
+*/
+	void AddChild (Object* object);
+
+	SpaceGroup const *FindSpaceGroup ();
+/*!
+Reinitialize a CrystalDoc instance. Used when loading a file in an already existing document.
+*/
+	void Reinit ();
+
 protected:
 /*!
 Initialize a new CrystalDoc instance.
 */
 	void Init ();
 /*!
-Reinitialize a CrystalDoc instance. Used when loading a file in an already existing document.
-*/
-	void Reinit ();
-/*!
 @param node: the xmlNode containing the serialized view.
 
 Loads a view from a XML document. This methd must be overrided by applications supporting multiple views.
@@ -269,12 +292,15 @@ List of the views of the document.
 */
 	std::list <CrystalView *> m_Views;
 
-#ifdef HAVE_OPENBABEL_2_2
-/*!
-The space group associated with the lattice.
+/*!GetSpaceGroup()
+@return the space group associated with the lattice.
 */
-	OpenBabel::SpaceGroup const *m_SpaceGroup;
-#endif
+
+GCU_RO_PROP (std::string, NameCommon);
+GCU_RO_PROP (std::string, NameSystematic);
+GCU_RO_PROP (std::string, NameMineral);
+GCU_RO_PROP (std::string, NameStructure);
+GCU_RO_PROP (SpaceGroup const *, SpaceGroup);
 };
 
 extern gchar const *LatticeName[];
