@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * curvedarrowtool.cc 
  *
- * Copyright (C) 2004-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -58,13 +58,39 @@ bool gcpCurvedArrowTool::OnClicked ()
 	gccv::ArrowHeads arrow_head = m_Full? gccv::ArrowHeadFull: gccv::ArrowHeadLeft;
 	gccv::BezierArrow *arrow = NULL; // make g++ happy
 	m_SourceAux = NULL;
+	m_pData->UnselectAll ();
 	if (m_pObject)
 		switch (m_pObject->GetType ()) {
 		case gcu::AtomType: {
 			gcp::Atom *atom = static_cast <gcp::Atom *> (m_pObject);
 			if (!AllowAsSource (atom))
 			    return false;
-			return false; // TODO: implement and return true
+			// find if there is an explicit electron pair or single electron and use it as source
+			// TODO
+			// find the most probable bond or the nearest atom
+			if (atom->GetBondsNumber () > 0) {
+				gcp::Bond *bond;
+				std::map< gcu::Atom *, gcu::Bond * >::iterator it;
+				double x, y;
+				atom->GetCoords (&x, &y);
+				x *= pTheme->GetZoomFactor ();
+				y *= pTheme->GetZoomFactor ();
+				if (atom->GetBondsNumber () == 1 || (x == m_x0 && y == m_y0)) // use first bond
+					m_Target = atom->GetFirstBond (it); // FIXME: check if the bond can ba a target
+				else {
+					double angle = 2 * M_PI; // something larger that anything we can get
+					for (bond = static_cast <gcp::Bond *> (atom->GetFirstBond (it)); bond; bond = static_cast <gcp::Bond *> (atom->GetNextBond (it))) {
+						m_Target = bond;
+					}
+				}
+				m_Item = arrow = new gccv::BezierArrow (m_pView->GetCanvas ());
+				AtomToAdjBond ();
+			} else {
+				// try to find a possible atom target
+				// TODO: implement and return true
+				m_Item = arrow = new gccv::BezierArrow (m_pView->GetCanvas ());
+				break;
+			}
 			break;
 		}
 		case gcu::BondType: {
@@ -88,10 +114,12 @@ bool gcpCurvedArrowTool::OnClicked ()
 			}
 			break;
 		}
-	arrow->SetShowControls (true);
-	arrow->SetLineWidth (pTheme->GetArrowWidth ());
-	arrow->SetLineColor (gcp::AddColor);
-	arrow->SetHead (arrow_head);
+	if (arrow) {
+		arrow->SetShowControls (true);
+		arrow->SetLineWidth (pTheme->GetArrowWidth ());
+		arrow->SetLineColor (gcp::AddColor);
+		arrow->SetHead (arrow_head);
+	}
 	return true;
 }
 
@@ -99,9 +127,6 @@ void gcpCurvedArrowTool::OnDrag ()
 {
 	if (!m_Item || !m_pObject)
 		return;	// this should not occur
-	double x0 = 0., y0 = 0., x1 = 0., y1 = 0., x2 = 0., y2 = 0., x3 = 0., y3 = 0., l, dx, dy;
-	gcp::Document *pDoc = m_pView->GetDoc ();
-	gcp::Theme *pTheme = pDoc->GetTheme ();
 	gccv::Item *item = m_pView->GetCanvas ()->GetItemAt (m_x, m_y);
 	if (item == m_Item) {
 		// TODO: implement moving the control points
@@ -124,6 +149,24 @@ void gcpCurvedArrowTool::OnDrag ()
 				}
 				break;
 			}
+			case gcu::AtomType: {
+				gcp::Atom *atom = static_cast <gcp::Atom *> (m_pObject);
+				if (!m_Target) {
+					std::map< gcu::Atom *, gcu::Bond * >::iterator it;
+					m_Target = atom->GetFirstBond (it); // TODO: be more intelligent
+					if (m_Target)
+						AtomToAdjBond ();
+					return;
+				} else {
+					gcp::Bond *bond = dynamic_cast <gcp::Bond *> (m_Target);
+					if (bond)
+						AtomToAdjBond ();
+					else
+						m_Target = NULL;
+					return; // TODO: implement
+				}
+				break;
+			}
 			default:
 				m_Target = NULL;
 				return;	// TODO: add more types
@@ -162,64 +205,15 @@ void gcpCurvedArrowTool::OnDrag ()
 					m_Target = NULL;
 					return;
 				}
-				double x, y, x_, y_;
-				gcp::Atom *start = static_cast <gcp::Atom *> (bond->GetAtom (0)),
-						  *end = static_cast <gcp::Atom *> (bond->GetAtom (1));
-				start->GetCoords (&x, &y);
-				end->GetCoords (&x_, &y_);
 				m_Target = bond;
-				// convert to canvas coordinates
-				x *= m_dZoomFactor;
-				y *= m_dZoomFactor;
-				x_ *= m_dZoomFactor;
-				y_ *= m_dZoomFactor;
 				switch (m_pObject->GetType ()) {
 				case gcu::BondType: {
-					if (static_cast <gcp::Bond *> (m_pObject)->GetAtom (start) == NULL) {
-						gcp::Atom *buf = start;
-						start = end;
-						end = buf;
-						// also exchange coordinates
-						x0 = x;
-						x = x_;
-						x_ = x0;
-						x0 = y;
-						y = y_;
-						y_ = y0;
-					}
-					// start is the common atom
-					x0 = m_CPx0;
-					y0 = m_CPy0;
-					x1 = x0 + m_CPx1;
-					y1 = y0 + m_CPy1;
-					x3 = (x + x_) / 2.;
-					y3 = (y + y_) / 2.;
-					// 
-					dx = y_ -y;
-					dy = x - x_;
-					l = hypot (dx, dy);
-					if (m_CPx1 * dy - m_CPy1 * dx > 0.) {
-						dx = -dx;
-						dy = -dy;
-					}
-					dx /= l;
-					dy /= l;
-					x3 += dx * pTheme->GetPadding ();
-					y3 += dy * pTheme->GetPadding ();
-					l += pTheme->GetArrowHeadA ();
-					m_CPx2 = dx * l;
-					m_CPy2 = dy * l;
-					x2 = x3 + m_CPx2;
-					y2 = y3 + m_CPy2;
-					m_SourceAux = NULL;
-					break;
+					BondToAdjBond ();
+					return;
 				}
 				case gcu::AtomType: {
-					gcp::Atom *atom = static_cast <gcp::Atom *> (cur);
-					if (!AllowAsTarget (atom)) {
-						m_Target = NULL;
-						return;
-					}
+					AtomToAdjBond ();
+					return;
 				}
 				default:
 					m_Target = NULL;
@@ -241,7 +235,11 @@ void gcpCurvedArrowTool::OnDrag ()
 						BondToAdjAtom ();
 					else
 						BondToAtom ();
+					break;
 				}
+				case gcu::AtomType:
+					AtomToAtom ();
+					break;
 				default:
 					m_Target = NULL;
 					return;
@@ -253,9 +251,10 @@ void gcpCurvedArrowTool::OnDrag ()
 				return;	// TODO: add more types
 			}
 		}
-	} else
+	} else {
 		m_Target = NULL;
-	arrow->SetControlPoints (x0, y0, x1, y1, x2, y2, x3, y3);
+		arrow->SetControlPoints (0., 0., 0., 0., 0., 0., 0., 0.);
+	}
 }
 
 void gcpCurvedArrowTool::OnMotion ()
@@ -338,12 +337,26 @@ bool gcpCurvedArrowTool::AllowAsSource (gcp::Atom *atom)
 {
 	if (!atom->HasAvailableElectrons (m_Full))
 	    return false;
+	std::set <gcu::Object *>::iterator i;
+	gcu::Object* obj = atom->GetFirstLink (i);
+	if (obj && obj->GetType () == gcp::MechanismArrowType) {
+		if (m_Full)
+			return false;
+		// for homolytic cleavage, the bond must be the source for both arrows
+		gcp::MechanismArrow *arrow = static_cast <gcp::MechanismArrow *> (obj);
+		if (arrow->GetPair () || arrow->GetSource () != atom)
+			return false;
+		// only one arrow should be there
+		obj = atom->GetNextLink (i);
+		if (obj && obj->GetType () == gcp::MechanismArrowType)
+			return false;
+	}
 	return true;
 }
 
-bool gcpCurvedArrowTool::AllowAsTarget (gcp::Atom *atom)
+bool gcpCurvedArrowTool::AllowAsTarget (G_GNUC_UNUSED gcp::Atom *atom)
 {
-	return true;
+	return (atom == m_pObject)? false: true;
 }
 
 bool gcpCurvedArrowTool::AllowAsSource (gcp::Bond *bond)
@@ -397,10 +410,79 @@ bool gcpCurvedArrowTool::AllowAsTarget (gcp::Bond *bond)
 
 void gcpCurvedArrowTool::AtomToAdjBond ()
 {
+	double x0 = 0., y0 = 0., x1 = 0., y1 = 0., x2 = 0., y2 = 0., x3 = 0., y3 = 0., l, dx, dy;
+	gcp::Theme *pTheme = m_pView->GetDoc ()->GetTheme ();
+	gcp::Bond *bond = static_cast <gcp::Bond *> (m_Target);
+	gcp::Atom *start = static_cast <gcp::Atom *> (bond->GetAtom (0)),
+			  *end = static_cast <gcp::Atom *> (bond->GetAtom (1));
+	if (end == m_pObject) {
+		end = start;
+		start = static_cast <gcp::Atom *> (m_pObject);
+	}
+	start->GetCoords (&x0, &y0);
+	end->GetCoords (&x1, &y1);
+	// convert to canvas coordinates
+	x0 *= m_dZoomFactor;
+	y0 *= m_dZoomFactor;
+	x1 *= m_dZoomFactor;
+	y1 *= m_dZoomFactor;
+	dx = y1 - y0;
+	dy = x0 - x1;
+	l = hypot (dx, dy);
+	dx /= l;
+	dy /= l;
+	x3 = (x0 + x1) / 2 + dx * pTheme->GetPadding ();
+	y3 = (y0 + y1) / 2 + dy * pTheme->GetPadding ();
+	l /= 2.;
+	m_CPx1 = m_CPx2 = l * dx;
+	m_CPy1 = m_CPy2 = l * dy;
+	double a = atan2 (-m_CPy1, m_CPx1) * 180. / M_PI;
+	start->GetPosition (a, x0, y0);
+	// convert to canvas coordinates
+	m_CPx0 = x0 *= pTheme->GetZoomFactor ();
+	m_CPy0 = y0 *= pTheme->GetZoomFactor ();
+	x1 = x0 + m_CPx1;
+	y1 = y0 + m_CPy1;
+	x2 = x3 + m_CPx1;
+	y2 = y3 + m_CPy1;
+	static_cast <gccv::BezierArrow *> (m_Item)->SetControlPoints (x0, y0, x1, y1, x2, y2, x3, y3);
 }
 
 void gcpCurvedArrowTool::AtomToAtom ()
 {
+	double x0 = 0., y0 = 0., x1 = 0., y1 = 0., x2 = 0., y2 = 0., x3 = 0., y3 = 0., l, dx, dy;
+	gcp::Theme *pTheme = m_pView->GetDoc ()->GetTheme ();
+	gcp::Atom *start = static_cast <gcp::Atom *> (m_pObject),
+			  *end = static_cast <gcp::Atom *> (m_Target);
+	start->GetCoords (&x0, &y0);
+	end->GetCoords (&x3, &y3);
+	x0 *= m_dZoomFactor;
+	y0 *= m_dZoomFactor;
+	x3 *= m_dZoomFactor;
+	y3 *= m_dZoomFactor;
+	dx = x3 - x0;
+	dy = y3 - y0;
+	l = hypot (dx, dy);
+	dx /= l;
+	dy /= l;
+	l = pTheme->GetBondLength () * m_dZoomFactor;
+	m_CPx1 = dy * l;
+	m_CPy1 = -dx * l;
+	double angle = -atan2 (m_CPy1, m_CPx1) * 180. / M_PI;
+	start->GetPosition (angle, x0, y0);
+	m_CPx0 = x0 *= m_dZoomFactor;
+	m_CPy0 = y0 *= m_dZoomFactor;
+	x1 = x0 + m_CPx1;
+	y1 = y0 + m_CPy1;
+	m_CPx2 = -dx * l;
+	m_CPy2 = -dy * l;
+	angle = -atan2 (m_CPy2, m_CPx2) * 180. / M_PI;
+	end->GetPosition (angle, x3, y3);
+	x3 *= m_dZoomFactor;
+	y3 *= m_dZoomFactor;
+	x2 = x3 + m_CPx2;
+	y2 = y3 + m_CPy2;
+	static_cast <gccv::BezierArrow *> (m_Item)->SetControlPoints (x0, y0, x1, y1, x2, y2, x3, y3);
 }
 
 void gcpCurvedArrowTool::BondToAdjAtom ()
@@ -485,8 +567,68 @@ void gcpCurvedArrowTool::BondToAdjAtom ()
 
 void gcpCurvedArrowTool::BondToAdjBond ()
 {
+	double x0 = 0., y0 = 0., x1 = 0., y1 = 0., x2 = 0., y2 = 0., x3 = 0., y3 = 0., l, dx, dy;
+				double x, y, x_, y_;
+	gcp::Theme *pTheme = m_pView->GetDoc ()->GetTheme ();
+	gcp::Bond *bond = static_cast <gcp::Bond *> (m_Target);
+	gcp::Atom *start = static_cast <gcp::Atom *> (bond->GetAtom (0)),
+			  *end = static_cast <gcp::Atom *> (bond->GetAtom (1));
+	start->GetCoords (&x, &y);
+	end->GetCoords (&x_, &y_);
+	// convert to canvas coordinates
+	x *= m_dZoomFactor;
+	y *= m_dZoomFactor;
+	x_ *= m_dZoomFactor;
+	y_ *= m_dZoomFactor;
+	if (static_cast <gcp::Bond *> (m_pObject)->GetAtom (start) == NULL) {
+		gcp::Atom *buf = start;
+		start = end;
+		end = buf;
+		// also exchange coordinates
+		x0 = x;
+		x = x_;
+		x_ = x0;
+		x0 = y;
+		y = y_;
+		y_ = x0;
+	}
+	// start is the common atom, x and y its coordinates
+	x0 = m_CPx0;
+	y0 = m_CPy0;
+	x1 = x0 + m_CPx1;
+	y1 = y0 + m_CPy1;
+	x3 = (x + x_) / 2.;
+	y3 = (y + y_) / 2.;
+	// 
+	dx = y_ - y;
+	dy = x - x_;
+	l = hypot (dx, dy);
+	if ((m_CPx1 * (y0 - y) - m_CPy1 * (x0 - x)) * (dx * (y3 - y) - dy * (x3 - x)) > 0.) {
+		dx = -dx;
+		dy = -dy;
+	}
+	dx /= l;
+	dy /= l;
+	x3 += dx * pTheme->GetPadding ();
+	y3 += dy * pTheme->GetPadding ();
+	l /= 2.;
+	l += pTheme->GetArrowHeadA ();
+	m_CPx2 = dx * l;
+	m_CPy2 = dy * l;
+	x2 = x3 + m_CPx2;
+	y2 = y3 + m_CPy2;
+	m_SourceAux = NULL;
+	static_cast <gccv::BezierArrow *> (m_Item)->SetControlPoints (x0, y0, x1, y1, x2, y2, x3, y3);
 }
 
 void gcpCurvedArrowTool::BondToAtom ()
 {
 }
+
+void gcpCurvedArrowTool::ElectronToAdjBond ()
+{
+}
+
+void gcpCurvedArrowTool::ElectronToAtom ()
+{
+}
