@@ -27,6 +27,11 @@
 
 #include <gcp/tool.h>
 
+namespace gcp {
+	class Atom;
+	class Bond;
+};
+
 class gcpCurvedArrowTool: public gcp::Tool
 {
 public:
@@ -37,9 +42,26 @@ public:
 	void OnDrag ();
 	void OnMotion ();
 	void OnRelease ();
+	void OnLeaveNotify ();
+
+private:
+	bool AllowAsSource (gcp::Atom *atom);
+	bool AllowAsTarget (gcp::Atom *atom);
+	bool AllowAsSource (gcp::Bond *bond);
+	bool AllowAsTarget (gcp::Bond *bond);
+
+	void AtomToAdjBond ();
+	void AtomToAtom ();
+	void BondToAdjAtom ();
+	void BondToAdjBond ();
+	void BondToAtom ();
+	void ElectronToAdjBond ();
+	void ElectronToAtom ();
 
 private:
 	bool m_Full; // if false use half heads
+	gcu::Object *m_Target, *m_SourceAux;
+	double m_CPx0, m_CPy0, m_CPx1, m_CPy1, m_CPx2, m_CPy2; // x and y deltas from ends to central control points
 };
 
 #endif	//GCHEMPAINT_CURVED_ARROW_TOOL_H
