@@ -25,6 +25,8 @@
 #include "config.h"
 #include "texttool.h"
 #include <gcp/application.h>
+#include <gcp/atom.h>
+#include <gcp/bond.h>
 #include <gcp/document.h>
 #include <gcp/fragment.h>
 #include <gcp/settings.h>
@@ -42,6 +44,10 @@
 
 using namespace gcu;
 extern GtkTargetEntry const text_targets[];
+namespace gcp {
+extern xmlDocPtr pXmlDoc;
+};
+
 
 GtkTargetEntry const text_targets[] = {
 	{(char *) GCHEMPAINT_ATOM_NAME,  0, gcp::GCP_CLIPBOARD_NATIVE},
@@ -60,8 +66,8 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 {
 	m_Active = NULL;
 	m_bUndo = true;
-	m_CurNode = m_InitNode = NULL;
-	m_Strikethrough = false;
+	m_CurNode = m_InitNode = m_GroupNode = NULL;
+	m_Strikethrough = gccv::TextDecorationNone;
 	m_FontDesc = NULL;
 	gcp::Theme *pTheme = gcp::TheThemeManager.GetTheme ("Default");
 	m_FamilyName = pTheme->GetTextFontFamily ();
@@ -70,10 +76,11 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 	m_Stretch = pTheme->GetFontStretch ();
 	m_Variant = pTheme->GetFontVariant ();
 	m_Size = pTheme->GetFontSize ();
-	m_Underline = PANGO_UNDERLINE_NONE;
+	m_Underline = gccv::TextDecorationNone;
 	m_Rise = 0;
-	m_Color = RGBA_BLACK;
+	m_Color = GO_COLOR_BLACK;
 	m_Position = gccv::Normalscript;
+	m_Group = NULL;
 }
 
 gcpTextTool::~gcpTextTool ()
@@ -120,6 +127,9 @@ bool gcpTextTool::OnClicked ()
 			BuildTagsList ();
 		else
 			UpdateTagsList ();
+		m_Group = m_pObject->GetGroup ();
+		if (m_pView->GetDoc ()->GetCurrentOperation () == NULL && m_Group)
+			m_GroupNode = m_Group->Save (gcp::pXmlDoc);
 	}
 	return true;
 }
@@ -169,7 +179,7 @@ bool gcpTextTool::OnKeyPress (GdkEventKey* event)
 				BuildTagsList ();
 				return true;
 			case GDK_u:
-				gtk_combo_box_set_active (m_UnderlineBox, ((m_Underline == PANGO_UNDERLINE_SINGLE)? PANGO_UNDERLINE_NONE: PANGO_UNDERLINE_SINGLE));
+				gtk_combo_box_set_active (m_UnderlineBox, ((m_Underline == gccv::TextDecorationDefault)? 0: 1));
 				return true;
 			case GDK_b:
 				m_Weight = (m_Weight == PANGO_WEIGHT_NORMAL)? PANGO_WEIGHT_BOLD: PANGO_WEIGHT_NORMAL;
@@ -197,6 +207,35 @@ bool gcpTextTool::OnKeyPress (GdkEventKey* event)
 				BuildTagsList ();
 				return true;
 			}
+			case GDK_l:
+				if (m_Active)
+					m_Active->SetJustification (GTK_JUSTIFY_LEFT, true);
+				return true;
+			case GDK_r:
+				if (m_Active)
+					m_Active->SetJustification (GTK_JUSTIFY_RIGHT, true);
+				return true;
+			case GDK_j:
+				if (m_Active)
+					m_Active->SetJustification (GTK_JUSTIFY_FILL, true);
+				return true;
+			case GDK_m:
+				if (m_Active)
+					m_Active->SetJustification (GTK_JUSTIFY_CENTER, true);
+				return true;
+			case GDK_w:
+				if (m_Active) {
+					double w = m_Active->GetInterline ();
+					if (w == 0.)
+						return true;
+					w -= 1.;
+					m_Active->SetInterline ((w > 0.? w: 0.), true);
+				}
+				return true;
+			case GDK_W:
+				if (m_Active) 
+					m_Active->SetInterline (m_Active->GetInterline () + 1., true);
+				return true;
 			default:
 				break;
 			}
@@ -207,6 +246,13 @@ bool gcpTextTool::OnKeyPress (GdkEventKey* event)
 	return false;
 }
 
+bool gcpTextTool::OnKeyRelease (GdkEventKey* event)
+{
+	if (m_Active)
+		m_Active->OnKeyReleased (event);
+	return true;
+}
+
 void gcpTextTool::Activate ()
 {
 	if (!m_Active)
@@ -247,34 +293,64 @@ bool gcpTextTool::Unselect ()
 	xmlBufferPtr endbuf = xmlBufferCreate ();
 	xmlNodeDump (initbuf, m_pApp->GetXmlDoc (), m_InitNode, 0, 0);
 	xmlNodeDump (endbuf, m_pApp->GetXmlDoc (), m_CurNode, 0, 0);
-	if (strcmp ((char*) initbuf->content, (char*) endbuf->content))
-	{
-		char* initval = (char*) xmlNodeGetContent (m_InitNode);
-		char* endval = (char*) xmlNodeGetContent (m_CurNode);
-		gcp::Operation *pOp = NULL;
-		if ((initval && strlen (initval))) {
-			if (endval && strlen (endval)) {
+	gcp::Operation *pOp = NULL;
+	if (strcmp ((char*) initbuf->content, (char*) endbuf->content)) {
+		if (m_Group) {
+			pOp = m_pView->GetDoc ()->GetCurrentOperation ();
+			if (!pOp) {
 				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
-				pOp->AddNode (m_InitNode, 0);
-				pOp->AddNode (m_CurNode, 1);
-				m_CurNode = m_InitNode = NULL;
-			} else {
-				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
-				pOp->AddNode (m_InitNode);
-				m_InitNode = NULL;
+				pOp->AddNode (m_GroupNode, 0);
 			}
-		} else if (endval && strlen (endval)) {
-			pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
-			pOp->AddNode (m_CurNode);
-			m_CurNode = NULL;
+			pOp->AddNode (m_Group->Save (gcp::pXmlDoc), 1);
+			m_Group = NULL;
+			m_GroupNode = NULL;
+		} else {
+			char* endval = (char*) xmlNodeGetContent (m_CurNode);
+			char* initval = (char*) xmlNodeGetContent (m_InitNode);
+			gcp::Fragment *fragment = dynamic_cast <gcp::Fragment *> (pObj);
+			map<Atom*, Bond*>::iterator i;
+			gcp::Bond *pBond = (fragment)? reinterpret_cast <gcp::Bond *> (fragment->GetAtom ()->GetFirstBond (i)): NULL;
+			if ((initval && strlen (initval))) {
+				if (endval && strlen (endval)) {
+					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
+					pOp->AddNode (m_InitNode, 0);
+					pOp->AddNode (m_CurNode, 1);
+					if (pBond) {
+						pOp->AddObject (pBond, 0);
+						pOp->AddObject (pBond, 1);
+					}
+					m_CurNode = m_InitNode = NULL;
+				} else {
+					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
+					pOp->AddNode (m_InitNode);
+					if (pBond)
+						pOp->AddObject (pBond);
+					m_InitNode = NULL;
+				}
+			} else if (endval && strlen (endval)) {
+				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
+				pOp->AddNode (m_CurNode);
+				m_CurNode = NULL;
+			}
+			if (initval)
+				xmlFree (initval);
+			if (endval)
+				xmlFree (endval);
 		}
-		if (initval)
-			xmlFree (initval);
-		if (endval)
-			xmlFree (endval);
 		if (pOp)
 			m_pView->GetDoc ()->PushOperation (pOp, m_bUndo);
 		m_bUndo = true;
+	} else {
+		if (m_Group) {
+			if (m_GroupNode) {
+				xmlFree (m_GroupNode);
+				m_GroupNode = NULL;
+			}
+			m_Group = NULL;
+		}
+		pOp = m_pView->GetDoc ()->GetCurrentOperation ();
+		if (pOp)
+			m_pView->GetDoc ()->AbortOperation ();
 	}
 	xmlBufferFree (initbuf);
 	xmlBufferFree (endbuf);
@@ -287,8 +363,10 @@ bool gcpTextTool::Unselect ()
 		Object* pMol = pObj->GetMolecule ();	//if pObj is a fragment
 		if (pMol)
 			pObj = pMol;
+		gcu::Object *parent = pObj->GetParent ();
 		m_pView->GetDoc ()->Remove (pObj);
 		m_pView->GetDoc ()->AbortOperation ();
+		parent->EmitSignal (gcp::OnChangedSignal);
 	}
 	m_pView->GetDoc ()->FinishOperation ();
 	m_pView->GetDoc ()->GetWindow ()->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", m_pView->GetDoc ()->HasChildren ());
@@ -537,9 +615,9 @@ void gcpTextTool::UpdateTagsList ()
 	m_Variant = pTheme->GetFontVariant ();
 	m_Size = pTheme->GetFontSize ();
 	m_Rise = 0;
-	m_Underline = PANGO_UNDERLINE_NONE;
-	m_Strikethrough = false;
-	m_Color = RGBA_BLACK;
+	m_Underline = gccv::TextDecorationNone;
+	m_Strikethrough = gccv::TextDecorationNone;
+	m_Color = GO_COLOR_BLACK;
 	if (m_Active) {
 		unsigned index = GetIndex ();
 		std::list <gccv::TextTag *> const *tags = m_Active->GetTags ();
@@ -799,7 +877,7 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	m_StrikethroughSignal = g_signal_connect (G_OBJECT (m_StrikethroughBtn), "toggled", G_CALLBACK (on_strikethrough_toggled), this);
 	m_RiseButton = GTK_SPIN_BUTTON (builder->GetWidget ("rise"));
 	m_RiseSignal = g_signal_connect (G_OBJECT (m_RiseButton), "value-changed", G_CALLBACK (on_rise_changed), this);
-	m_ColorSelector = GO_SELECTOR (go_color_selector_new (RGBA_BLACK, RGBA_BLACK, "fore"));
+	m_ColorSelector = GO_SELECTOR (go_color_selector_new (GO_COLOR_BLACK, GO_COLOR_BLACK, "fore"));
 	go_color_selector_set_allow_alpha (m_ColorSelector, false);
 	m_ForeSignal = g_signal_connect (G_OBJECT (m_ColorSelector), "activate", G_CALLBACK (on_fore_color_changed), this);
 	gtk_widget_show (GTK_WIDGET (m_ColorSelector));
@@ -1006,7 +1084,20 @@ unsigned gcpTextTool::GetIndex ()
 
 void gcpTextTool::OnUnderlineChanged (unsigned underline)
 {
-	m_Underline = (PangoUnderline) underline;
+	switch (underline) {
+	default:
+		m_Underline = gccv::TextDecorationNone;
+		break;
+	case 1:
+		m_Underline = gccv::TextDecorationDefault;
+		break;
+	case 2:
+		m_Underline = gccv::TextDecorationDouble;
+		break;
+	case 3:
+		m_Underline = gccv::TextDecorationLow;
+		break;
+	}
 	BuildTagsList ();
 	if (m_Active) {
 		gccv::TextTagList l;
@@ -1017,7 +1108,7 @@ void gcpTextTool::OnUnderlineChanged (unsigned underline)
 
 void gcpTextTool::OnStriketroughToggled (bool strikethrough)
 {
-	m_Strikethrough = strikethrough;
+	m_Strikethrough = strikethrough? gccv::TextDecorationDefault: gccv::TextDecorationNone;
 	BuildTagsList ();
 	if (m_Active) {
 		gccv::TextTagList l;
